"""
    sphinxcontrib
    ~~~~~~~~~~~~~

    This package is a namespace package that contains all extensions
    distributed in the ``sphinx-contrib`` distribution.

    :copyright: (c) 2016 by Igor Kalnitsky.
    :license: BSD, see LICENSE for details.
"""

__import__("pkg_resources").declare_namespace(__name__)
